/*
Copyright 2006, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2006
*/

/** \file ScatterWeb.Configuration.h
 ** \ingroup System
 ** \brief The ScatterWeb.Configuration provides fuctions that allow you to
 ** programmatically access configuration settings.
 ** <p>
 ** The configuration consists of the struct config_t defined in
 ** ScatterWeb.Configuration.h. The variable Configuration is of that type and
 ** represents the current config in RAM and can be accessed directly everytime.
 ** <p>
 ** Initial loading is done by Configuration_init() and
 ** has to be called on initialization in main().
 ** <p>
 ** First time configuration:
 ** Uncomment the following definition of firstTimeConfig and it will get layed
 ** into Infomem (0x1000-0x1100, see .hex-File). Before Configuration_init()
 ** loads the regular config from EEPROM, it checks if the first int (which
 ** corresponds to the id) in Infomem is not 0xFFFF. If it is not, it copies the
 ** configuration from Infomem to EEPROM (clearing Infomem afterwards and
 ** loading the newly written data from EEPROM to Configuration).
 **
 ** @author        Marco Hutta
 ** @version       1.0
 ** @since         1.0
 **/  

#ifndef ScatterWeb_Configuration
#define ScatterWeb_Configuration

/**
 * Radio debug output.
 *
 * @see           #Comm_log(UINT8, const char*, ...) 
 * @see           #Configuration.boot_logLevel 
 * @since         1.0
 */
#define DEBUG_RADIO     (0x80)

/**
 * Dco adjusting debug output.
 *
 * @see           #Comm_log(UINT8, const char*, ...) 
 * @see           #Configuration.boot_logLevel 
 * @since         1.0
 */
#define DEBUG_DCO       (0x40)

/**
 * Comm_print() gets through.
 *
 * @see           #Comm_log(UINT8, const char*, ...) 
 * @see           #Configuration.boot_logLevel 
 * @since         1.0
 */
#define NO              (0x01)

/**
 * Comm_print(), Comm_log(LOW) get through.
 *
 * @see           #Comm_log(UINT8, const char*, ...) 
 * @see           #Configuration.boot_logLevel 
 * @since         1.0
 */
#define LOW             (0x02)

/**
 * Comm_print(), Comm_log(LOW), Comm_log(MEDIUM) get through.
 *
 * @see           #Comm_log(UINT8, const char*, ...) 
 * @see           #Configuration.boot_logLevel 
 * @since         1.0
 */
#define MEDIUM          (0x03)

/**
 * Comm_print(), Comm_log(LOW), Comm_log(MEDIUM), Comm_log(HIGH) get through.
 *
 * @see           #Comm_log(UINT8, const char*, ...) 
 * @see           #Configuration.boot_logLevel 
 * @since         1.0
 */
#define HIGH            (0x04)

/**
 * All get through.
 *
 * @see           #Comm_log(UINT8, const char*, ...) 
 * @see           #Configuration.boot_logLevel 
 * @since         1.0
 */
#define VERBOSE         (0x05)

/**
 * If set device allows reprogramming itself.
 *
 * @see           #Configuration.firmwareFlags 
 * @since         1.0
 */
#define FF_PROGRAMMABLE            (0x01)

/**
 * If set dco adjusting will be enabled.
 *
 * @see           #Configuration.firmwareFlags 
 * @since         1.0
 */
#define FF_DCOCHECKER              (0x02)

/**
 * Configuration of the device
 * The configuration is automatically loaded into Configuration at boot time.
 * <ul>
 * <li>id - Id of this MSP device
  * <li>transmitPower - Power for transmittting, set via Net_setTxPower at boot time.
 * <li>boot_logLevel - Comm_logLevel is initialized with that value.
 * <li>panicFlags - Controls panic behaviour.
 * <li>firmwareFlags - Other bits for firmware behaviour.
 * <li>chipconConfig - configuration of chipcon
 * <li>freqOffset - offset of frequency
 * </ul>
 *  
 * @since         1.0
 */
typedef struct {
  UINT16 id;
  UINT8 transmitPower;       
  UINT8 boot_logLevel;
  UINT8 firmwareFlags;
} config_t;

/**
 * Copy of the Configuration
 *
 * @since         1.0
 */
extern config_t Configuration;

/**
 * Initializes configuration settings for the configuration section
 *
 * @return        <code>True</code> always.
 * @since         1.0
 */
bool Configuration_init();

/**
 * Prints configuration settings for the configuration section
 *
 * @return        <code>True</code> always.
 * @since         1.0
 */
bool Configuration_print(); 

#endif
